@@ -169,14 +169,15 @@ int __kmp_str_buf_vprint(kmp_str_buf_t *buffer, char const *format,
 
     // Try to format string.
     {
-/* On Linux* OS Intel(R) 64, vsnprintf() modifies args argument, so vsnprintf()
-   crashes if it is called for the second time with the same args. To prevent
-   the crash, we have to pass a fresh intact copy of args to vsnprintf() on each
-   iteration.
+      /* On Linux* OS Intel(R) 64, vsnprintf() modifies args argument, so
+         vsnprintf() crashes if it is called for the second time with the same
+         args. To prevent the crash, we have to pass a fresh intact copy of args
+         to vsnprintf() on each iteration.
 
-   Unfortunately, standard va_copy() macro is not available on Windows* OS.
-   However, it seems vsnprintf() does not modify args argument on Windows* OS.
-*/
+         Unfortunately, standard va_copy() macro is not available on Windows*
+         OS. However, it seems vsnprintf() does not modify args argument on
+         Windows* OS.
+      */
 
 #if !KMP_OS_WINDOWS
       va_list _args;
@@ -403,7 +404,7 @@ void __kmp_str_loc_free(kmp_str_loc_t *loc) {
 int __kmp_str_eqf( // True, if strings are equal, false otherwise.
     char const *lhs, // First string.
     char const *rhs // Second string.
-    ) {
+) {
   int result;
 #if KMP_OS_WINDOWS
   result = (_stricmp(lhs, rhs) == 0);
@@ -447,7 +448,7 @@ int __kmp_str_eqf( // True, if strings are equal, false otherwise.
 char *__kmp_str_format( // Allocated string.
     char const *format, // Format string.
     ... // Other parameters.
-    ) {
+) {
   va_list args;
   int size = 512;
   char *buffer = NULL;
@@ -546,7 +547,7 @@ void __kmp_str_split(char *str, // I: String to split.
                      char delim, // I: Character to split on.
                      char **head, // O: Pointer to head (may be NULL).
                      char **tail // O: Pointer to tail (may be NULL).
-                     ) {
+) {
   char *h = str;
   char *t = NULL;
   if (str != NULL) {
@@ -570,7 +571,7 @@ char *__kmp_str_token(
     char *str, // String to split into tokens. Note: String *is* modified!
     char const *delim, // Delimiters.
     char **buf // Internal buffer.
-    ) {
+) {
   char *token = NULL;
 #if KMP_OS_WINDOWS
   // On Windows* OS there is no strtok_r() function. Let us implement it.
@@ -652,7 +653,7 @@ void __kmp_str_to_size( // R: Error code.
     size_t *out, // O: Parsed number.
     size_t dfactor, // I: The factor if none of the letters specified.
     char const **error // O: Null if everything is ok, error message otherwise.
-    ) {
+) {
 
   size_t value = 0;
   size_t factor = 0;
@@ -751,7 +752,7 @@ void __kmp_str_to_uint( // R: Error code.
     char const *str, // I: String of characters, unsigned number.
     kmp_uint64 *out, // O: Parsed number.
     char const **error // O: Null if everything is ok, error message otherwise.
-    ) {
+) {
   size_t value = 0;
   int overflow = 0;
   int i = 0;
